// Copyright 2017 The Johns Hopkins University Applied Physics Laboratory.
// Licensed under the MIT License. See LICENSE.txt in the project root for full license information.


#pragma once

#include <iostream>
#include <chrono>

namespace chrono = std::chrono;


namespace eopcc {

/// A class for handling unbuffed command line annoucements
class Announcer {
private:
  bool print_steps_;
  int num_steps_;
  int curr_step_;

  int curr_level_;
  int num_levels_;

  chrono::steady_clock::time_point start_point_;

  virtual void Print(const std::string& output) {
    std::cout << output;
  }

  virtual void Return() {
    std::cout << std::endl;
  }

  virtual void Flush() {
     std::cout << std::flush;
  }

  virtual void PrintProgressBar(int full, int total) {
    #ifndef WIN32
    Print("\t[");
    if (full >= total) {
      std::string full_str(full, (char)176u);
      Print(full_str);
    } else {
      std::string full_str(full, (char)176u);
      std::string empty_str(total-full, ' ');
      Print(full_str + empty_str);
    }

    Print("]\r");
    Flush();
    #else
    Say("[" +std::to_string(full) + "/" + std::to_string(total) + "]");
    #endif
  }

public:
  Announcer() : print_steps_(false) {}

  void SetNumSteps(int num_steps) {
    if (num_steps > 0) {
      num_steps_ = num_steps;
      print_steps_ = true;
      curr_step_ = 0;
    } else {
      print_steps_ = false;
    }
  }

  void PrintStep() {
    if (print_steps_) {
      curr_step_++;
     Print("[" + std::to_string(curr_step_) + "/" + std::to_string(num_steps_) + "] ");
    }
  }

  void Say(const std::string& announcement) {
    Print(announcement);
    Return();
  }

  void Title(const std::string& title) {
    std::string deco(10, '*');
    Say(deco + " " + title + " " + deco);
  }

  void StepStart(const std::string& start_announ) {
    PrintStep();
    Say(start_announ + "... ");
    start_point_ = chrono::steady_clock::now();
  }

  void SubSay(const std::string& subannoun) {
    Say("\t- " + subannoun);
  }

  void ProgressBarStart(int num_levels=100) {
    curr_level_ = 0;
    num_levels_ = num_levels;
    PrintProgressBar(curr_level_, num_levels_);
  }


  void ProgressBarUpdate(double fraction_complete) {
    int new_level = fraction_complete * num_levels_;

    if (new_level != curr_level_) {
      curr_level_ = new_level;
      PrintProgressBar(curr_level_, num_levels_);
    }
  }

  virtual void ProgressBarEnd() {
    PrintProgressBar(num_levels_, num_levels_);
    Say("");
  }

  void StepEnd(const std::string& end_announ="Done") {
    chrono::duration<double> span = chrono::duration_cast<chrono::duration<double>>(chrono::steady_clock::now() - start_point_);
    Say("\t" + end_announ + " [" + std::to_string(span.count()) + "s]");
  }
};


} // namespace eopcc
